use psxcdgen_ex::types::{layout::Layout, CDDesc, File, Directory};
use tool_helper::Error;

fn populate() -> Result<CDDesc, Error> {
    let mut desc   = CDDesc::new();
    let     file   = File::new("Planschi.jpg")?;
    let     file2  = File::new("Wuff.png")?;
    let     folder = {
        let mut folder     = Directory::new("Sub")?;
        let     sub_folder = {
            let mut folder = Directory::new("SubSub")?;

            folder.add_file(File::new("Blubb.bin")?);
            folder
        };

        folder.add_dir(sub_folder);
        folder.add_file(File::new("Schwimm.jpg")?);
        folder.add_file(File::new("Miau.png")?);
        folder
    };

    desc.add_dir(folder);
    desc.add_file(file);
    desc.add_file(file2);

    desc.calculate_lbas();
    Ok(desc)
}

fn run_main() -> Result<(), Error> {
    let desc = populate()?;

    println!("\n<== Planschbecken ==>");
    for element in desc.get_memory_layout().iter() {
        match element {
            Layout::SystemArea(_)               => println!("SystemArea:"),
            Layout::PVD(_)                      => println!("PVD:"),
            Layout::PathTables                  => println!("PathTables:"),
            Layout::Directory(dir)              => println!("Dir: {} @{}-{}", dir.borrow().name, dir.borrow().properties.lba, dir.borrow().properties.overwrite_size_bytes.unwrap_or(0)),
            Layout::File(file)                  => println!("File: {} @{}-{}", file.borrow(), file.borrow().properties.lba, file.borrow().properties.overwrite_size_bytes.unwrap_or(0)),
        }
    }
    Ok(())
}

fn main() {
    match run_main() {
        Ok(_)      => println!("\n<== Planschbecken ==>"),
        Err(error) => println!("{}", error)
    }
}