use super::*;

const CURRENT_DIR_NAME:&'static str = "/x00";
const PARENT_DIR_NAME:&'static str  = "/x01";

pub struct PathTableMember {
    pub name:            String,
    pub parent_table_id: usize,
}

pub enum DirectoryRecordMember {
    Directory{name: String},
    File{name: String},
}

impl DirectoryRecordMember {
    pub fn get_name(&self) -> &String {
        match self {
            DirectoryRecordMember::Directory{name} => name,
            DirectoryRecordMember::File{name}      => name,
        }
    }
}

pub fn collect_path_table_member(root: SharedPtr<Directory>) -> Vec<PathTableMember> {
    fn collect_path_table_for(collection: &mut Vec<PathTableMember>, dirs: &Vec<SharedPtr<Directory>>, parent_id: usize) {
        let mut cur_dirs = Vec::new();

        for dir in dirs {
            cur_dirs.push(PathTableMember{name: dir.borrow().name.as_string().unwrap_or("".to_owned()), parent_table_id: parent_id});
        }

        cur_dirs.sort_by(|a, b| {
            a.name.cmp(&b.name)
        });

        collection.append(&mut cur_dirs);

        for dir in dirs {
            collect_path_table_for(collection, &dir.borrow().dirs, parent_id + 1);
        }
    }
    let mut collection = Vec::new();
    let     root       = root.borrow();
    
    collection.push(PathTableMember{name: CURRENT_DIR_NAME.to_owned(), parent_table_id: 1});
    collect_path_table_for(&mut collection, &root.dirs, 1);
    collection
}

pub fn collect_directory_record_member(files: &Vec<SharedPtr<File>>, dirs: &Vec<SharedPtr<Directory>>) -> Vec<DirectoryRecordMember> {
    let mut collection = Vec::new();

    collection.push(DirectoryRecordMember::Directory{name: CURRENT_DIR_NAME.to_owned()});
    collection.push(DirectoryRecordMember::Directory{name: PARENT_DIR_NAME.to_owned()});
    for file in files {
        let file = file.borrow();
        if !file.properties.is_hidden {
            if let Some(name) = file.name.as_string() {
                collection.push(DirectoryRecordMember::File{name});
            }
        }
    }

    for dir in dirs {
        if !dir.borrow().properties.is_hidden {
            if let Some(name) = dir.borrow().name.as_string() {
                collection.push(DirectoryRecordMember::Directory{name});
            }
        }
    }

    collection.sort_by(|a, b| {a.get_name().cmp(b.get_name())});
    collection
}
